"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const sequence_1 = require("../sequence");
const set_1 = require("./set");
const assertion_test_1 = require("../equality/assertion.test");
const codable_1 = require("../codable");
const optional_1 = require("../optional");
(0, vitest_1.describe)("Set Tests", () => {
    (0, vitest_1.test)("Iterable", () => {
        const s = (0, set_1.set)([4, 1, 3, 2]);
        let actual = (0, sequence_1.seq)();
        for (const element of s) {
            actual = actual.append(element);
        }
        (0, vitest_1.expect)(actual.sorted()).toEqual((0, sequence_1.seq)([1, 2, 3, 4]));
    });
    (0, vitest_1.test)("集合の要素が合致してた場合、equalsがtrueを返す", () => {
        const s1 = (0, set_1.set)([4, 2, 3, 2, 1]);
        const s2 = (0, set_1.set)([3, 3, 1, 2, 4]);
        (0, assertion_test_1.assertEquals)(s1, s2);
    });
    (0, vitest_1.test)("countで要素数を取得できる", () => {
        const s = (0, set_1.set)([1, 2, 3]);
        (0, vitest_1.expect)(s.count).toBe(3);
    });
    (0, vitest_1.test)("要素空がない場合、isEmptyがfalseを返す", () => {
        (0, vitest_1.expect)((0, set_1.set)([1]).isEmpty).toBeFalsy();
    });
    (0, vitest_1.test)("要素が空の場合isEmptyがtrueを返す", () => {
        (0, vitest_1.expect)((0, set_1.set)([]).isEmpty).toBeTruthy();
    });
    (0, vitest_1.test)("要素空がない場合、nonEmptyがtrueを返す", () => {
        (0, vitest_1.expect)((0, set_1.set)([1]).nonEmpty).toBeTruthy();
    });
    (0, vitest_1.test)("要素が空の場合nonEmptyがfalseを返す", () => {
        (0, vitest_1.expect)((0, set_1.set)([]).nonEmpty).toBeFalsy();
    });
    (0, vitest_1.test)("insert値を追加できる", () => {
        const s = (0, set_1.set)([1, 2, 3]);
        const expected = (0, set_1.set)([1, 2, 3, 4]);
        const actual = s.insert(4);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("mapで値を変換できる", () => {
        const s = (0, set_1.set)([1, 2, 3]);
        const expected = (0, set_1.set)([2, 4, 6]);
        const actual = s.map((n) => n * 2);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("flatMapで値を変換して平らにできる", () => {
        const sequence = (0, set_1.set)([1, 2, 3]);
        const expected = (0, set_1.set)([1, 11, 2, 12, 3, 13]);
        const actual = sequence.flatMap((n) => (0, set_1.set)([n, n + 10]));
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("compactMapでnone値を除外して変換できる", () => {
        const s = (0, set_1.set)((0, optional_1.some)(1), (0, optional_1.some)(2), (0, optional_1.none)(), (0, optional_1.some)(4));
        const expected = (0, set_1.set)(3, 6, 12);
        const actual = s.compactMap((n) => n * 3);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("filter", () => {
        const s = (0, set_1.set)([1, 2, 3]);
        const expected = (0, set_1.set)([1, 3]);
        const actual = s.filter((n) => n % 2 === 1);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("reduce", () => {
        const s = (0, set_1.set)([1, 2, 3]);
        const expected = 6;
        const actual = s.reduce(0, (acc, n) => acc + n);
        (0, vitest_1.expect)(actual).toBe(expected);
    });
    (0, vitest_1.test)("全ての要素がtrueのときにeveryがtrueを返す", () => {
        const s = (0, set_1.set)([1, 2, 3]);
        const actual = s.every((n) => n > 0);
        (0, vitest_1.expect)(actual).toBeTruthy();
    });
    (0, vitest_1.test)("どれか一つの要素がfalseのときにeveryがfalseを返す", () => {
        const s = (0, set_1.set)([1, -1, 3]);
        const actual = s.every((n) => n > 0);
        (0, vitest_1.expect)(actual).toBeFalsy();
    });
    (0, vitest_1.test)("どれか1つの要素がtrueのときにsomeがtrueを返す", () => {
        const s = (0, set_1.set)([-1, 2, 3]);
        const actual = s.some((n) => n < 0);
        (0, vitest_1.expect)(actual).toBeTruthy();
    });
    (0, vitest_1.test)("どれか一つの要素がfalseのときにsomeがfalseを返す", () => {
        const s = (0, set_1.set)([1, 2, 3]);
        const actual = s.some((n) => n < 0);
        (0, vitest_1.expect)(actual).toBeFalsy();
    });
    (0, vitest_1.test)("numberでsortが可能", () => {
        const s = (0, set_1.set)([2, 1, 3]);
        const expected = (0, sequence_1.seq)([1, 2, 3]);
        const actual = s.sorted();
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("引数でsortが可能", () => {
        const s = (0, set_1.set)([2, 1, 3]);
        const expected = (0, sequence_1.seq)([3, 2, 1]);
        const actual = s.sorted((a, b) => a > b);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    class N extends codable_1.Codable {
        constructor(value) {
            super();
            this.value = value;
        }
        compare(compared) {
            return this.value < compared.value;
        }
    }
    (0, vitest_1.test)("Comparable Objectでsortが可能", () => {
        const s = (0, set_1.set)([new N(2), new N(1), new N(3)]);
        const expected = (0, sequence_1.seq)([new N(1), new N(2), new N(3)]);
        const actual = s.sorted();
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("maxを取得できる", () => {
        const s = (0, set_1.set)([new N(2), new N(1), new N(3)]);
        const expected = (0, optional_1.some)(new N(3));
        const actual = s.max();
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("minを取得できる", () => {
        const s = (0, set_1.set)([new N(2), new N(1), new N(3)]);
        const expected = (0, optional_1.some)(new N(1));
        const actual = s.min();
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("含むかを確認できる", () => {
        const s = (0, set_1.set)([new N(1), new N(2), new N(3)]);
        (0, vitest_1.expect)(s.contains(new N(2))).toBeTruthy();
        (0, vitest_1.expect)(s.contains(new N(4))).toBeFalsy();
    });
    (0, vitest_1.test)("findで検索できる", () => {
        const sequence = (0, sequence_1.seq)([1, 2, 3]);
        const expected = (0, optional_1.some)(2);
        const actual = sequence.find((n) => n === 2);
        (0, assertion_test_1.assertEquals)(actual, expected);
    });
    (0, vitest_1.test)("encodableに準拠", () => {
        const sequence = (0, set_1.set)([1, 2, 3]);
        const expected = [1, 2, 3];
        const actual = sequence.encode;
        (0, vitest_1.expect)(actual).toEqual(expected);
    });
    (0, vitest_1.test)("decodableに準拠", () => {
        const json = [{ value: 3 }, { value: 4 }, { value: 6 }];
        const expected = (0, set_1.set)([new N(3), new N(4), new N(6)]);
        const actual = (0, set_1.set)(new N(0)).decode(json);
        (0, vitest_1.expect)(actual).toEqual(expected);
    });
    (0, vitest_1.test)("intersectionで交差している要素のみを抽出する", () => {
        const s1 = (0, set_1.set)([1, 2, 3]);
        (0, assertion_test_1.assertEquals)(s1.intersection([2, 3, 4]), (0, set_1.set)([2, 3]));
    });
    (0, vitest_1.test)("unionで和集合を取得する", () => {
        const s1 = (0, set_1.set)([1, 2, 3]);
        (0, assertion_test_1.assertEquals)(s1.union([2, 3, 4]), (0, set_1.set)([1, 2, 3, 4]));
    });
    (0, vitest_1.test)("isSubsetで、引数が部分集合の場合にtrueを返す", () => {
        const s1 = (0, set_1.set)([1, 2, 3]);
        (0, vitest_1.expect)(s1.isSubset([2, 3])).toBeTruthy();
    });
    (0, vitest_1.test)("isSubsetで、引数が部分集合出ない場合にfalseを返す", () => {
        const s1 = (0, set_1.set)([1, 2, 3]);
        (0, vitest_1.expect)(s1.isSubset([2, 3, 4])).toBeFalsy();
    });
    (0, vitest_1.test)("exceptで差集合を返される", () => {
        const s1 = (0, set_1.set)([1, 2, 3, 4, 5]);
        (0, assertion_test_1.assertEquals)(s1.except([2, 3]), (0, set_1.set)([1, 4, 5]));
    });
});
//# sourceMappingURL=set.test.js.map